# This is the core JavaScript logic to handle the synchronization
# It will be injected into the page later
SYNC_JS = """
function setupSyncScroll(area1Selector, area2Selector, verse1Selector, verse2Selector) {
    const area1 = document.querySelector(area1Selector);
    const area2 = document.querySelector(area2Selector);

    // NiceGUI's ui.scroll_area is a Quasar q-scrollarea.
    // We need to find the actual scrollable DOM element,
    // which is the parent of the .q-scrollarea__content div.
    if (!area1 || !area2) {
        console.error('Scroll areas not found');
        return;
    }
    const scrollRoot1 = area1.querySelector('.q-scrollarea__content')?.parentElement;
    const scrollRoot2 = area2.querySelector('.q-scrollarea__content')?.parentElement;
    
    if (!scrollRoot1 || !scrollRoot2) {
        console.error('Scrollable content roots not found');
        return;
    }

    // Get all verse elements (the <vid> tags) and map them by their ID
    const verses1 = Array.from(area1.querySelectorAll(verse1Selector));
    const verses2 = Array.from(area2.querySelectorAll(verse2Selector));
    
    // We change from 'dataset.vid' to 'id' to match your HTML
    const verses1Map = new Map(verses1.map(v => [v.id, v]));
    const verses2Map = new Map(verses2.map(v => [v.id, v]));

    let timer1, timer2;
    let lastScroller = null; // Replaces 'scrollingFromSync'

    // Finds the topmost visible verse in a scrolling container
    function getTopVerse(scrollRoot, verses) {
        const scrollTop = scrollRoot.scrollTop;
        // We add a 5px buffer to be less sensitive
        const scrollThreshold = scrollTop + 5; 
        
        // Find the first verse whose top offset is at or just below the scroll top
        let topVerse = verses[0];
        for (let i = 0; i < verses.length; i++) {
            // We use the <vid> tag's parentElement (<verse>) for offsetTop
            // to get the top of the whole verse block.
            const verseBlock = verses[i].parentElement;
            if (verseBlock.offsetTop >= scrollThreshold) {
                // We've gone past it, so the previous one was the top one
                topVerse = verses[i-1] || verses[0];
                break;
            }
            topVerse = verses[i]; // Keep updating in case it's the last one
        }
        return topVerse;
    }

    // Scroll handler for Area 1
    scrollRoot1.addEventListener('scroll', () => {
        if (lastScroller === 'area2') return; // Ignore scrolls caused by area2
        clearTimeout(timer1);
        timer1 = setTimeout(() => { // Debounce to avoid lag
            lastScroller = 'area1'; // Mark area1 as the one scrolling
            const topVerse = getTopVerse(scrollRoot1, verses1);
            if (topVerse) {
                const vid = topVerse.id; // Use .id
                const targetElement = verses2Map.get(vid);
                if (targetElement) {
                    // Scroll to the top of the target's parent <verse> tag
                    const targetBlock = targetElement.parentElement;
                    scrollRoot2.scrollTop = targetBlock.offsetTop;
                    // Release the lock after a short delay
                    setTimeout(() => { lastScroller = null; }, 150); 
                } else {
                    lastScroller = null; // Release lock if no target
                }
            } else {
                 lastScroller = null; // Release lock if no verse
            }
        }, 50); // 50ms debounce
    });

    // Scroll handler for Area 2
    scrollRoot2.addEventListener('scroll', () => {
        if (lastScroller === 'area1') return; // Ignore scrolls caused by area1
        clearTimeout(timer2);
        timer2 = setTimeout(() => { // Debounce
            lastScroller = 'area2'; // Mark area2 as the one scrolling
            const topVerse = getTopVerse(scrollRoot2, verses2);
            if (topVerse) {
                const vid = topVerse.id; // Use .id
                const targetElement = verses1Map.get(vid);
                if (targetElement) {
                    // Scroll to the top of the target's parent <verse> tag
                    const targetBlock = targetElement.parentElement;
                    scrollRoot1.scrollTop = targetBlock.offsetTop;
                    // Release the lock after a short delay
                    setTimeout(() => { lastScroller = null; }, 150);
                } else {
                    lastScroller = null; // Release lock if no target
                }
            } else {
                 lastScroller = null; // Release lock if no verse
            }
        }, 50);
    });
}
"""

def get_sync_fx(tab1, tab2):
    return f"""        
        // We wait 500ms for all elements to be rendered and positioned
        // before we set up the listeners.
        setTimeout(() => {{
            // Note the new verse selectors!
            setupSyncScroll('.{tab1}', '.{tab1}', '.{tab1+"_chapter"} vid', '.{tab2+"_chapter"} vid');
        }}, 500);
    """


ORIGINAL_CSS = """
<style>
    /* Each verse acts as a container for word blocks */
    verse {
        display: block;
        margin-bottom: 20px;
        line-height: 1.3;
    }
    /* The interlinear word block container */
    .int {
        display: inline-block;
        vertical-align: top;
        text-align: center;
        margin: 0 4px 10px 4px;
        padding: 4px 8px;
        background: white;
        border-radius: 6px;
        box-shadow: 0 2px 4px rgba(0,0,0,0.05);
        border: 1px solid #eee;
        transition: background-color 0.2s; /* Smooth highlight transition */
    }
    /* Target all block elements inside .int to tighten their spacing */
    .int > *, .int > ref > * {
        line-height: 1.1;
        margin-top: 1px;
        margin-bottom: 1px;
    }
    /* Hover effects for interactive elements (even if JS isn't active here) */
    .int:hover {
        background-color: #f8fbff;
        border-color: #d6eaf8;
    }

    /* Transliteration Layers (Phonetic & SBL) - shared with Greek*/
    wsbl, wphono {
        direction: ltr;
        display: block;
        font-size: 0.8rem;
        color: #7f8c8d;
        font-style: italic;
    }
    /* Morphology Layer */
    wmorph {
        direction: ltr;
        display: block;
        font-family: monospace;
        font-size: 0.7rem;
        color: #27ae60;
        cursor: pointer;
    }
    wsn {
        direction: ltr;
        display: block;
        font-size: 0.7rem;
        color: #8e44ad;
        cursor: pointer;
    }
    /* Gloss (Literal Meaning) Layer */
    wgloss {
        direction: ltr;
        display: block;
        font-size: 0.85rem;
        color: #d35400;
    }
    /* Final Translation Layer */
    wtrans {
        direction: ltr;
        display: block;
        margin-top: 4px;
        padding-top: 3px;
        border-top: 1px solid #f0f0f0;
        font-size: 0.95rem;
        font-weight: bold;
        color: #2980b9;
        min-height: 1.2em; /* Ensures empty translations don't collapse the block */
    }

    /* UBA css */
    external {
        font-size: 80%;
    }
    red, z {
        color: red;
    }
    blu {
        color: blue;
        font-size: 80%;
    }
    points {
        color: gray;
        font-weight: bold;
        font-size: 80%;
    }
    bb {
        color: brown;
        font-weight: bold;
    }
    hp {
        color: brown;
        font-weight: bold;
        font-size: 80%;
    }
    highlight {
        font-style: italic;
    }
    transliteration {
        color: gray;
    }
    div.section, div.point {
        display: block;
        border: 1px solid green;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 5px;
        margin-bottom: 5px;
    }
    div.remarks {
        display: block;
        border: 1px solid gray;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 5px;
        margin-bottom: 5px;
    }
    div.bhs, div.hebch {
        direction: rtl;
    }
    div.info {
        margin-left: 5%;
        margin-right: 5%;
    }
    div.menu {
        margin-left: 2%;
        margin-right: 2%;
    }
    div.vword {
        border: 1px solid #F5B041;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.translation {
        border: 1px solid #9B59B6;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.ew {
        margin-left: 5%;
        margin-right: 5%;
        font-size: 110%;
        display: inline-block;
    }
    div.mr {
        margin-left: 100;
    }
    div.nav {
        margin-left: 5%;
        margin-right: 5%;
    }
    div.refList {
        display: inline;
    }
    /* css for linguistic annotations */
                
    div.bhp, div.bhw, div.w, div.int {
        display: inline-block;
        text-align: center;
    }
    div.int {
        vertical-align: text-top;
    }
    div.bhc {
        direction: rtl;
        border: 1px solid #9B59B6;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.bhp {
        border: 1px solid gray;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.bhw {
        border: 1px solid #F5B041;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    clid {
        color: #641E16;
        font-weight: bold;
    }
    mclid {
        color: #641E16;
        font-style: italic;
    }
    connector {
        color: #9B59B6;
    }
    ckind, ctyp, crela {
        color: #00008B;
        display: inline-block;
    }
    ptyp {
        text-decoration: underline;
        color: gray;
    }
    pfunction {
        color: #3498DB;
        font-size: 90%;
    }
    det {
        font-weight: bold;
    }
    undet {
        font-style: italic;
    }
    prela {
        display: none;
    }
    hbint, gntint, gloss {
        direction: ltr;
        vertical-align: super;
        display: inline-block;
        color: #D35400;
    }
    gntint, gloss {
        font-size: 90%;
    }
    cllevel {
        color: #9B59B6;
    }
    clinfo {
        color: #641E16;
        font-weight: bold;
    }
    subclinfo {
        color: #641E16;
        font-weight: bold;
        font-size: 80%;
    }
    funcinfo {
        color: #3498DB;
        font-size: 90%;
    }
    wordid {
        text-decoration: underline;
        font-size: 80%;
        color: gray;
    }
    cit, clt, cst, cbhs, cbsb, cleb {
        display: block;
    }
    clt, cbsb {
        color: #3498DB;
    }
    cst, cleb {
        color: #00008B;
    }
    cbsb, cleb {
        direction: ltr;
    }
    div.cltrans {
        margin-left: 10px;
        margin-right: 10px;
        display: block;
        border: 1px solid #00008B;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.wrap {
        display: inline-block;
    }
    cl {
        display: table;
    }
    div.c {
        vertical-align: top;
        text-align: left;
        border: 1px solid #9B59B6;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.p {
        display: inline-block;
        vertical-align: top;
        text-align: left;
        border: 1px solid gray;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }
    div.w, div.int {
        border: 1px solid #F5B041;
        border-radius: 5px;
        padding: 2px 5px;
        margin-top: 3px;
        margin-bottom: 3px;
    }

    /* css for clause segmentation */

    div.bhsa {
        direction: rtl;
        border-right: 5px solid #F5B041;
        margin-left: 5%;
        margin-right: 5%;
        padding: 5px 10px 5px 10px;
    }
    div.e {
        border-left: 5px solid #F5B041;
        margin-left: 5%;
        margin-right: 5%;
        padding: 5px 10px 5px 10px;
    }
</style>
"""


ORIGINAL_JS = """
<script>
    // MOCK W3.JS (Polyfill to avoid external dependency)
    var w3 = {
        addStyle: function(selector, prop, value) {
            document.querySelectorAll(selector).forEach(function(el) {
                 el.style.setProperty(prop, value);
            });
        }
    };

    // Variable used in original script for host interoperability

    function hl0(id, cl, sn) {
        if (cl != '') {
            w3.addStyle('.c'+cl,'background-color','');
        }
        if (sn != '') {
            w3.addStyle('.G'+sn,'background-color','');
        }
        if (id != '') {
            var focalElement = document.getElementById('w'+id);
            if (focalElement != null) {
                focalElement.style.background='';
            }
        }
    }

    function hl1(id, cl, sn) {
        if (cl != '') {
            w3.addStyle('.c'+cl,'background-color','PAPAYAWHIP');
        }
        if (sn != '') {
            w3.addStyle('.G'+sn,'background-color','#E7EDFF');
        }
        if (id != '') {
            var focalElement = document.getElementById('w'+id);
            if (focalElement != null) {
                focalElement.style.background='#C9CFFF';
            }
        }
    }
</script>
"""